/** Ein abgeschlossener (encapsulation) Suchbaum
 * implementiert mit dem State- und Sigleton-Pattern.
 */
class BinSuchbaum
{
  /** Die Wurzel Knoten
   * Ein Baum ist durch die Wurzel und seine Kinder definiert 
   * Fr den leeren Baum ist die Wurzel durch den LeerenKnoten
   * reprsenteirt.
   */
  private AKnoten _wurzel;
  
  /** Erzeugt leerer Baum */
  public BinSuchbaum() {
    _wurzel = LeererKnoten.signleton();
  }
  
  /** Ist der Schlssel im Baum (Suche) 
    * @param k Schlssel
    */
  public boolean suchen(int k) {
    return _wurzel.suchen(k);
  }
  
  /** Fgt Schlssel ein 
   * @param k Schlssel
   */
  public void einfuegen(int k) {
    _wurzel.einfuegen(this, k);
  }
  
  /** Entfernt Schlsel
   * @param k Schlssel
   */
  public void entferne(int k) {
    _wurzel.entferne(this, k);
  }
  
  /** Ist Baum leer
   * @return true false leer
   */
  public boolean istLeer() {
    return _wurzel.istLeer();
  }
  
  /** Gibt den Baum in inorder aus. */
  public void printInOrder() {
    _wurzel.printInOrder();
  }
  
  /** Entfernt grsster Schlssel im Baum
   * wird fr lschen(k) bentigt. Darf nur auf
   * nichtleerem Baum aufgerufen werden.
   * @return grsster Schlssel im Baum
   */
  private int entferneGroesster() {
    return _wurzel.entferneGroesster(this);
  }
  
  /** Setzt Wurzel im Baum 
   * @k neue Wurzel
   */
  private void setzeWurzel(AKnoten k) {
    _wurzel = k;
  }
  
  
  /** Abstrakter Knoten
   * Suchbaum aggregiert diesen Knoten als Wurzel.
   * Die Abstrakte Klasse wird fr das State-Pattern
   * verwendet.
   * State-Pattern ermglicht eine einfache
   *  funktionsbezogene Implementierung.
   */
  static private abstract class AKnoten
  {
    /* Alle Operationen die auf den Knoten mglich sein sollen
      mssen hier definiert werden.
     */
    public abstract boolean suchen(int k);
    public abstract void einfuegen(BinSuchbaum b, int i);
    public abstract void entferne(BinSuchbaum b, int k);
    public abstract boolean istLeer();
    
    public abstract void printInOrder();
    
    public abstract int entferneGroesster(BinSuchbaum b);
  }
  
  /** Leerer Knoten
   * Diese Klasse reprsentiert den leeren Knoten.
   * Singleton Klasse -> es gibt nur eine Instanz,
   *   alle leeren Bume sind gleich.
   * Implementiert alle Operationen auf dem leeren Baum.
   */
  static private class LeererKnoten extends AKnoten
  {
    
    public void einfuegen(BinSuchbaum b, int i) {
      // TODO
    }
    
    public boolean suchen(int k) {
      return false;
    }
    
    public void entferne(BinSuchbaum b, int k) {
      // TODO
    }
    
    public boolean istLeer() {
      // TODO
    }
    
    public void printInOrder() {
      // TODO
    }
    
    /* nicht definiert auf leerem Knoten */
    public int entferneGroesster(BinSuchbaum b) {
      System.exit(0); //Dieser Fall darf nie auftreten.
      // Knnte auch mittels Exception gelst werden.
      return 0;
    }
    
    private static AKnoten _einmalig = null;
    
    public static AKnoten signleton() {
      if (_einmalig == null) {
        _einmalig = new LeererKnoten();
      }
      return _einmalig;
    }
  }
  
  /** NichtLeererKnoten
   * Zustand nicht leerer Knoten. Beinhaltet den
   * Schlssel.
   * Implementiert alle Operationen auf einem Knoten
   */
  static private class NichtLeererKnoten extends AKnoten
  {
    // Felder
    /** Das linke Kind */
    private BinSuchbaum _links;
    /** Das rechte Kind */
    private BinSuchbaum _rechts;
    
    /** Der Schlssel */
    private int _schluessel;
    
    
    // Konstruktor
    public NichtLeererKnoten(int schluessel) {
      _schluessel = schluessel;
      _links = new BinSuchbaum();
      _rechts = new BinSuchbaum();
    }
    
    // Operationen
    public void einfuegen(BinSuchbaum b, int k) {
      // TODO
    }
    
    public void entferne(BinSuchbaum b, int k) {
      // TODO
    }
    public boolean suchen(int k) {
      if (k == _schluessel) { return true; }
      if (k < _schluessel) { return _links.suchen(k); }
      else { return _rechts.suchen(k); }
    }

    public boolean istLeer() {
      // TODO
    }
    
    public void printInOrder() {
      // TODO
    }
    
    public int entferneGroesster(BinSuchbaum b) {
      if (_rechts.istLeer()) {
        b.setzeWurzel(_links._wurzel);
        return _schluessel;
      } else {
        return _links.entferneGroesster();
      }
    }
  }
}